<?php
/**
 * @file
 * Refund functionalities for Commerce Square.
 */

/**
 * Refund a given amount from a given transaction.
 *
 * @param object $transaction
 *   The commerce_payment_transaction entity object to refund.
 * @param int $amount
 *   The integer amount to refund.
 */
function commerce_square_refund_transaction($transaction, $amount) {
  if (!is_numeric($amount)) {
    throw new \SquareConnect\ApiException(t('Attempted a refund with non-numeric amount.'));
  }
  if ($amount != round($amount)) {
    throw new \SquareConnect\ApiException(t('Attempted a refund with non-integer amount.'));
  }
  if ($amount > $transaction->amount) {
    throw new \SquareConnect\ApiException(t('Attempted a refund with an amount larger than the original transaction amount.'));
  }

  libraries_load('square');
  $payment_method = commerce_payment_method_instance_load($transaction->instance_id);
  list($transaction_id, $tender_id) = explode('|', $transaction->remote_id);
  $refund_request = new \SquareConnect\Model\CreateRefundRequest([
    'idempotency_key' => uniqid(),
    'tender_id' => $tender_id,
    'amount_money' => new \SquareConnect\Model\Money([
      'amount' => (int) $amount,
      'currency' => $transaction->currency_code,
    ]),
    'reason' => t('Refunded through store backend'),
  ]);

  $mode = $payment_method['settings']['mode'];
  \SquareConnect\Configuration::getDefaultConfiguration()->setAccessToken($payment_method['settings'][$mode . '_access_token']);
  $transaction_api = new \SquareConnect\Api\TransactionsApi();
  $result = $transaction_api->createRefund(
      $payment_method['settings'][$mode . '_location_id'],
      $transaction_id,
      $refund_request
      );

  $refund_transaction = commerce_payment_transaction_new('commerce_square', $transaction->order_id);
  $refund_transaction->instance_id = $transaction->instance_id;
  $refund_transaction->amount = -$amount;
  $refund_transaction->currency_code = $transaction->currency_code;
  $refund_transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
  $remote_transaction = $result->getRefund();
  $tender_id = $remote_transaction->getTenderId();
  $refund_transaction->remote_id = $remote_transaction->getId() . '|' . $tender_id;
  $refund_transaction->order_id = $transaction->order_id;
  $refund_transaction->message = t('Manual refund');
  commerce_payment_transaction_save($refund_transaction);
}

/**
 * Refund form callback.
 */
function commerce_square_refund_form($form, &$form_stat, $transaction) {
  // Only positive amount transactions can be refunded.
  if ($transaction->amount <= 0) {
    return [];
  }
  $form = [];

  $form['amount'] = [
    '#title' => t('Amount to refund'),
    '#type' => 'textfield',
    '#field_suffix' => $transaction->currency_code,
    '#element_validate' => ['commerce_square_refund_amount_validate'],
  ];
  $form['actions'] = [
    '#type' => 'actions',
  ];
  $form['actions']['submit'] = [
    '#type' => 'submit',
    '#value' => 'Next',
  ];
  return $form;
}

/**
 * Refund form validation callback.
 */
function commerce_square_refund_amount_validate($element, &$form_state, $form) {
  if ($element['#value'] <= 0) {
    form_error($element, t('Only positive amounts can be refunded.'));
  }
  $transaction = $form_state['build_info']['args'][0];
  $currency_code = $transaction->currency_code;
  $max_decimal = commerce_currency_amount_to_decimal($transaction->amount, $currency_code);
  if ($element['#value'] > $max_decimal) {
    $vars = [
      '@amount' => $max_decimal,
      '@currency' => $currency_code,
    ];
    form_error($element, t('You cannot refund more than @amount @currency.', $vars));
  }
}

/**
 * Form submit callback to redirect to refund confirmation.
 */
function commerce_square_refund_form_submit($form, $form_state) {
  $transaction = $form_state['build_info']['args'][0];
  $currency_code = $transaction->currency_code;
  $amount = commerce_currency_decimal_to_amount($form_state['values']['amount'], $currency_code);
  drupal_goto('admin/commerce/orders/' . $transaction->order_id . '/payment/' . $transaction->transaction_id . '/refund/' . $amount);
}

/**
 * Confirm form callback to refund order.
 */
function commerce_square_refund_confirm_form($form, &$form_state, $transaction, $amount) {
  $vars = [
    '@amount' => commerce_currency_amount_to_decimal($amount, $transaction->currency_code),
    '@currency' => $transaction->currency_code,
  ];
  return confirm_form([], t('Are you sure you want to refund @amount @currency?', $vars), 'admin/commerce/orders/' . $transaction->order_id . '/payment', t('This action cannot be undone.'), t('Refund'));
}

/**
 * Confirm form submit callback to refund an order.
 */
function commerce_square_refund_confirm_form_submit($form, $form_state) {
  libraries_load('square');
  $transaction = $form_state['build_info']['args'][0];
  $amount = $form_state['build_info']['args'][1];
  $vars = [
    '@transaction' => $transaction->transaction_id,
    '@amount' => $form_state['build_info']['args'][1],
  ];
  try {
    commerce_square_refund_transaction($transaction, $amount);
    drupal_set_message(t('Amount successfully refunded'), 'status');
    watchdog('commerce_square', 'Refund successful. Transaction: @transaction, amount: @amount.', $vars);
  }
  catch (Exception $e) {
    $vars['@message'] = $e->getMessage();
    drupal_set_message(t('Refund failed. Square message: @message', $vars), 'warning');
    watchdog('commerce_square', 'Refund failed. Transaction: @transaction, amount: @amount, message: @message', $vars);
  }

  // Redirect to payments page.
  drupal_goto('admin/commerce/orders/' . $transaction->order_id . '/payment');
}
